#ifndef C_ACTOR_H
#define C_ACTOR_H

#include <string>
#include <allegro.h>

// Forward declarations
class c_game;
class c_sprite;

using namespace std;

/* Class: c_actor
 * 
 * Base class to encapsulates any actor (character, cursor etc) in the game.
 */
class c_actor
{
protected:
	c_game   *m_p_game;
	float     m_x;           // Actor's x position
	float     m_y;           // ... y
	float     m_xvel;        // Actor's velocity vector, x component
	float     m_yvel;        // ... y component
	int       m_total_anims; // Total number of defined animationd for this actor
	c_sprite *m_sprite;      // The actor's sprite object
	string    m_anims[16];   // Array of animation identifiers (like "IDLE", "SPWN" etc)
	
	void _init(void);
public:
	c_actor();
	
	/* Overloaded constructor
	 * Arguments: -
	 *  - game:     pointer to the mai c_game object.
	 *  - dat_file: file name of the actor data file.
	 */
	c_actor(c_game *game, string dat_file);
	
	~c_actor();
	
	/* Loads an actor data file
	 * Arguments: -
	 *  - dat_file: file name of the actor data file.
	 */
	void load_dat_file(string dat_file);
	
	/* Getter and setter for actor animation (by string identifier) */
	string get_anim(void);
	bool set_anim(string anim_ID);
	
	/* Virtual method
	 * Can be overloaded in derived class to allow drawing routines specific
	 * to derived class.
	 * Draws actor's sprite to a BITMAP.
	 * Arguments: -
	 *  - bmp_dest: pointer to a BITMAP to draw to.
	 */
	virtual void draw(BITMAP *bmp_dest);
	
	/* Virtual method
	 * Can be overloaded in derived class to allow specific update routines.
	 * Moves the actor based on the velocity vector.
	 */
	virtual void update(void);
	
	/* Getters and setters for x, y, velocity vector */
	void set_x(float x);
	void set_y(float y);
	float get_x(void);
	float get_y(void);
	void set_xvel(float x_vel);
	void set_yvel(float y_vel);
	float get_xvel(void);
	float get_yvel(void);
	
	/* Return: the actor's sprite */
	c_sprite* get_sprite(void);
};

#endif
