#ifndef C_BITMAP_FACTORY_H
#define C_BITMAP_FACTORY_H

/*
 * The purpose of the c_bitmap_factory class is to provide a consistent
 * and memory-efficient way of loading bitmaps.
 * 
 * One instance of c_bitmap_factory should be created in the application.
 * 
 * When a BITMAP needs to be loaded, call c_bitmap_factory::get_bitmap().
 * 
 * get_bitmap() uses the filename to identify the bitmap uniquely. It
 * searches through a linked list of BITMAPs to see if the BITMAP has
 * already been loaded. If it has, it assigns the loaded BITMAP* to
 * pp_bmp. If it hasn't, it loads it from the file, adds it to the list
 * and assigns it to pp_bmp.
 */

#include <allegro.h>
#include <string>

using namespace std;

/* struct to provide a linked list of BITMAPS */
typedef struct _bmplist
{
	string filename;
	BITMAP *p_bmp;
	struct _bmplist *p_next;
} bmplist;

/* Class: c_bitmap_factory
 * 
 * See above for description
 */
class c_bitmap_factory
{
private:
	bmplist *m_p_bmphead; // Head of the bmplist linked list
public:
	c_bitmap_factory();
	~c_bitmap_factory();
	
	/* Gets a bitmap identified by the file name
	 * Arguments: -
	 *  - filename: file name of the bitmap to get.
	 *  - pp_bmp:   pointer to a pointer to a BITMAP, will be assigned to the retrieved BITMAP.
	 * Return: true if bitmap was successfully assigned to pp_bmp, false on failure.
	 */
	bool get_bitmap(string filename, BITMAP **pp_bmp);
};

#endif
