#ifndef C_CHARACTER_ACTOR_H
#define C_CHARACTER_ACTOR_H

#include "c_actor.h"
#include <string>

/* Class:  c_character_actor
 * Parent: c_actor
 * 
 * Base class for encapsulating any actor (enemy, player) in the game. Builds
 * upon c_actor for sprite animation handling, position, velocity etc.
 */
class c_character_actor : public c_actor
{
protected:
	float m_speed;  // How fast the character actor will move
	float m_health; // The character's health
public:
	c_character_actor();
	
	/* Overloaded constructor
	 * Arguments: -
	 *  - game:     pointer to the main c_game object.
	 *  - dat_file: file name of the character actor data file
	 */
	c_character_actor(c_game *game, string dat_file);
	
	~c_character_actor();
	
	/* Getters and setters for speed and health */
	void set_speed(float speed);
	float get_speed(void);
	void set_health(float health);
	float get_health(void);
	
	/* Function to handle a character hit (by a projectile etc).
	 * Arguments: -
	 *  - hit_power: the emount by which the health will be reduced.
	 * Return: true if health <= 0 (i.e. dead), false otherwise.
	 */
	bool hit(float hit_power);
};

#endif
