#ifndef C_JUKEBOX_H
#define C_JUKEBOX_H

#include <string>
#include "c_music.h"

/* The number of defined jukebox tracks (-1 for JBT_NONE which is not a real track) */
#define JUKEBOX_NUM_TRACKS 3

/* The jukebox track types: -
 *  - JBT_INTRO:  The intro (main menu) track.
 *  - JBT_MAIN_1: Gameplay track 1.
 *  - JBT_MAIN_2: Gameplay track 2.
 *  - JBT_NONE:   Gameplay, no music.
 */
enum JUKEBOX_TRACK { JBT_INTRO, JBT_MAIN_1, JBT_MAIN_2, JBT_NONE };

/* Class: c_jukebox
 * 
 * Handles a selection of c_music objects allowing just one to be played at once
 */
class c_jukebox
{
private:
	c_music *m_a_tracks[JUKEBOX_NUM_TRACKS]; // The c_music tracks
	JUKEBOX_TRACK m_curr_track;              // The currently playing track
	
	void _init(void);
	
public:
	c_jukebox();
	~c_jukebox();
	
	/* Loads a track
	 * Arguments: -
	 *  - track:    the track type to load (see enum above).
	 *  - filename: the file name of the RIFF Wave file containing the track.
	 * Return: true on success, false on failure.
	 */
	bool load_track(JUKEBOX_TRACK track, string filename);
	
	/* Plays a track
	 * Arguments: -
	 *  - track:     the track type to play (see enum above).
	 *  - f_restart: if true, the track will be restarted if it's the currently
	 *               playing track, if false the track will continue playing.
	 *               If the track is different to the current, this has no
	 *               effect.
	 */
	void play_track(JUKEBOX_TRACK track, bool f_restart);
	
	/* Return: the currently playing track */
	JUKEBOX_TRACK get_current_track(void);
};

#endif
