#ifndef C_MENU_H
#define C_MENU_H

#include <string>
#include <allegro.h>
#include "c_game.h"
#include "c_actor.h"

/* The maximum number of items a menu can have */
#define MENU_MAX_ITEMS        10

/* The initial y offset from the top used when drawing the menu */
#define MENU_INITIAL_OFFSET_Y 10

/* The y padding of each item in the menu */
#define MENU_ITEM_PADDING_Y   15

using namespace std;

/* Class: c_menu
 * 
 * Handles a game menu: drawing, keyboard/mouse handling etc
 */
class c_menu
{
private:
	c_game *m_p_game;
	string m_a_items[MENU_MAX_ITEMS]; // The menu items (strings)
	int m_item_count;
	int m_curr_item;                  // The currently selected item
	c_actor *m_p_actor_heading;       // The menu heading (actor, therefore sprite)
	c_actor *m_p_actor_cursor;        // The cursor actor (showing currently selected menu item)
	string m_message;                 // The message shown below the menu items
	BITMAP *m_p_backdrop;             // The menu background image
	
	void _init(void);
	
	/* Internal method to get the rectangular bounds of an item
	 * Arguments: -
	 *  - item_id: the menu item ID for which the bounds are to be found.
	 *  - p_x1:    pointer to an integer to accept the left bound value.
	 *  - p_y1:    ...top.
	 *  - p_x2:    ...right.
	 *  - p_y2:    ...bottom.
	 */
	void get_item_bounds(int item_id, int *p_x1, int *p_y1, int *p_x2, int *p_y2);
	
public:
	/* Overloaded constructor
	 * Argument: -
	 *  - game: pointer to the main c_game object.
	 */
	c_menu(c_game *game);
	
	~c_menu();
	
	/* Loads the heading actor (sprite)
	 * Argument: -
	 *  - filename: the file name of the actor data file.
	 */
	void load_actor_heading(string filename);
	
	/* Loads the menu message to be displayed below the items
	 * Argument: -
	 *  - filename: the file name of the text file to use as a source of the message.
	 */
	void load_message(string filename);
	
	/* Return: the heading actor */
	c_actor* get_actor_heading(void);
	
	/* Return: the number of items in the menu */
	int get_item_count(void);
	
	/* Adds an item to the menu
	 * Argument: -
	 *  - item_str: the string to use for the item.
	 * Return: the new item ID, or -1 on error.
	 */
	int add_item(string item_str);
	
	/* Draws the menu to a BITMAP
	 * Argument: -
	 *  - bmp: pointer to the BITMAP to draw to.
	 */
	void draw(BITMAP *bmp);
	
	/* Updates the menu (updates actors, handles mouse/keboard etc) */
	void update(void);
	
	/* Loads a background image for the menu, otherwise background will be black
	 * Argument: -
	 *  - filename: file name of the bitmap to load.
	 */
	void load_backdrop(string filename);
};

#endif
