#ifndef C_PARtICLE_H
#define C_PARTICLE_H

#include "allegro.h"

/* Class: c_particle
 * 
 * Encapsulates a single particle as part of a particle system
 */
class c_particle
{
private:
	bool m_f_dead;
	int m_col;      // The particle colour (result of makecol(r, g, b))
	char m_size;    // Radius of particle, 1 is a single pixel
	float m_x;
	float m_y;
	float m_xvel;
	float m_yvel;
	float m_damp_x; // How much m_xvel will be reduced by each update...
	float m_damp_y; // ...and m_yvel.
	int m_lifespan; // How many frames the particle will remain alive
	int m_age;      // The current particle age (if >= lifespan, it dies)
	c_particle *m_p_next; // Next pointer to establish linked list of particles
public:
	
	/* Overloaded constructor
	 * Arguments: -
	 *  - r, g, b:        RGB colour.
	 *  - size:           radius of particle, 1 is a single pixel.
	 *  - x, y:           the starting x and y position.
	 *  - xvel, yvel:     the particle's velocity vector.
	 *  - damp_x, damp_y: the damping applied to the velocity vector.
	 *  - lifespan:       for how many frames the particle will remain alive.
	 */
	c_particle(unsigned char r, unsigned char g, unsigned char b, char size, float x, float y, float xvel, float yvel, float damp_x, float damp_y, int lifespan);
	
	~c_particle();
	
	/* Updates the particle (moves it, increases age etc). */
	void update(void);
	
	/* Draws ths particle to a BITMAP
	 * Argument: -
	 *  - bmp_dest: pointer to the BITMAP to draw to.
	 */
	void draw(BITMAP *bmp_dest);
	
	/* Return: true if particle is not ative, false if it is active */
	bool is_dead(void);
	
	/* Getter and setter for particles linked list */
	void set_next(c_particle *next);
	c_particle* get_next(void);
};

#endif
