#ifndef C_PARTICLE_SYSTEM_H
#define C_PARTICLE_SYSTEM_H

#include "c_particle.h"
#include "allegro.h"

#define PI 3.1415926536

/* The cone angle of wall debris particles */
#define PST_WALL_DEBRIS_ANGLE_DEGREES 70

/* The initial speed of wall debris particles */
#define PST_WALL_DEBRIS_VELOCITY      50

/* The cone angle of blood particles */
#define PST_BLOOD_ANGLE_DEGREES       30

/* The initial speed of blood particles */
#define PST_BLOOD_VELOCITY             2

/* The particle system types: -
 *  - PST_WALL_DEBRIS:      spawned when a projectile hits a wall.
 *  - PST_BLOOD:            spawned when the player or an enemy is hit.
 *  - PST_PROJECTILE_TRAIL: unused currently.
 */
enum PARTICLE_SYSTEM_TYPE { PST_WALL_DEBRIS, PST_BLOOD, PST_PROJECTILE_TRAIL };

/* Class: c_particle_system
 * 
 * Handles a system of particles that behave in a way defined accordig to PARTICLE_SYSTEM_TYPE.
 */
class c_particle_system
{
private:
	PARTICLE_SYSTEM_TYPE m_pst;
	bool m_f_active;
	float m_x;
	float m_y;
	float m_vec_dir_x; // The base velocity vector (particles will inherit this plus random variation), x component...
	float m_vec_dir_y; // ... y component.
	c_particle *m_p_head_particle; // Head of the particle linked list.
	
	/* Internal method to spawn particles for the system */
	void spawn_particles(void);
public:
	/* Overloaded constructor
	 * Arguments: -
	 *  - pst:  the particle system type (as above enum).
	 *  - x, y: the origin of the particle system (particles will be emitted from this point).
	 *  - vec_dir_x, vec_dir_y: the particle system's base velocity vector.
	 */
	c_particle_system(PARTICLE_SYSTEM_TYPE pst, float x, float y, float vec_dir_x, float vec_dir_y);
	
	~c_particle_system();
	
	/* Updates the particle system (moves particles, checks particle life etc) */
	void update(void);
	
	/* Draws the particle system to a BiTMAP
	 * Argument: -
	 *  - bmp_dest: pointer to the BITMAP to draw to.
	 */
	void draw(BITMAP *bmp_dest);
	
	/* Indicates whether the particle system is active (i.e. all particles are still alive).
	 * Return: true if active, false if not
	 */
	bool is_active(void);
	
	/* Return: the particle system type */
	PARTICLE_SYSTEM_TYPE get_type(void);
};

#endif
