#ifndef C_PROJECTILE_H
#define C_PROJECTILE_H

#include "c_actor.h"
#include <string>

/* The type of damage the projectile does: -
 *  - DT_PLAYER: the projectile will damage the player but not enemies.
 *  - DT_ENEMY:  the projectile will damage enemies but not the player.
 *  - DT_BOTH:   the projectile will damage both enemies and the player.
 */
enum DAMAGE_TYPE { DT_PLAYER, DT_ENEMY, DT_BOTH };

/* The state of the projectile: -
 *  - PS_NORMAL: Having been fired, moving through the world
 *  - PS_DYING:  Having just hit something.
 *  - PS_DEAD:   Dying anim has finished, projectile is inactive.
 */
enum PROJECTILE_STATE { PS_NORMAL, PS_DYING, PS_DEAD };

/* Class:  c_projectile
 * Parent: c_actor
 * 
 * Encapsulates a player or enemy projectile. Builds on c_actor for the sprite,
 * position, velocity etc.
 */
class c_projectile : public c_actor
{
private:
	bool             m_f_active;
	DAMAGE_TYPE      m_damage_type;
	PROJECTILE_STATE m_state;
	float            m_damage_power; // The amount of damage the projectile will inflict
	string           m_dat_file;     // Copy of the actor data file name
	
	/* Performs default initialisation */
	void _init(void);
public:
	c_projectile();
	
	/* Overloaded constructor
	 * Arguments: -
	 *  - game:     pointer to main c_game.
	 *  - dat_file: string containing the file name of the actor data file.
	 */
	c_projectile(c_game *game, string dat_file);
	
	~c_projectile();
	
	/* Overloaded c_actor::update()
	 * Updates the projectile (move it etc).
	 */
	void update(void);
	
	/* Overloaded c_actor::draw()
	 * Draws the projectile to a BITMAP.
	 * Arguments: -
	 *  - bmp_dest: pointer to the BITMAP to draw to.
	 */
	void draw(BITMAP *bmp_dest);
	
	/* Activates/deactivates the projectile */
	void activate(void);
	void deactivate(void);
	
	/* Indicates whether the projectile is active
	 * Return: true if active, false if inactive.
	 */
	bool is_active(void);
	
	/* Return: the name of the actor data file */
	string get_dat_file_name(void);
	
	/* Getters and setters for damage type, damage power, state */
	DAMAGE_TYPE get_damage_type(void);
	void set_damage_type(DAMAGE_TYPE dt);
	void set_damage_power(float power);
	float get_damage_power(void);
	PROJECTILE_STATE get_state(void);
	void set_state(PROJECTILE_STATE state);
};

#endif
