#include "c_sfx.h"

c_sfx::c_sfx()
{
	m_f_enabled = false;
	m_p_voice_list = NULL;
	for (int i = 0; i < SFX_SAMPLE_COUNT; i++)
		m_a_samples[i] = NULL;
	load_voices();
}

c_sfx::~c_sfx()
{
	voice_list *p_vl = m_p_voice_list, *p_temp = NULL;
	
	// Destroy all voices
	while (p_vl != NULL)
	{
		p_temp = p_vl;
		p_vl = p_vl->p_next;
		if (p_temp->voice_id > -1)
		{
			voice_stop(p_temp->voice_id);
			deallocate_voice(p_temp->voice_id);
		}
		delete p_temp;
	}
	
	// Destroy all samples
	for (int i = 0; i < SFX_SAMPLE_COUNT; i++)
	{
		if (m_a_samples[i] != NULL)
		{
			destroy_sample(m_a_samples[i]);
			m_a_samples[i] = NULL;
		}
	}
}

bool c_sfx::load_voices(void)
{
	bool f_result = true;
	SAMPLE *p_samp = NULL;
	voice_list *p_vl = NULL;
	
	// The number of voices to allocate for each sample. 0 means that no voice
	// will be allocated, rather sample will be played directly with play_sample().
	int a_voice_counts[SFX_SAMPLE_COUNT]     = {   0,   0,   0,   0,   0,   0,   0,   0 };
	
	// The priority of each sample type
	int a_voice_priorities[SFX_SAMPLE_COUNT] = { 192, 128, 192, 255, 192, 255, 255, 128 };
	
	// The volume of each sample type
	int a_voice_volumes[SFX_SAMPLE_COUNT]    = { 128, 128, 128, 128, 128, 128, 128, 128 };
	
	// Load all samples
	m_a_samples[SFX_PLAYER_FIRE]   = load_sample("sfx/player_fire.wav");
	m_a_samples[SFX_ENEMY_FIRE]    = load_sample("sfx/enemy_fire.wav");
	m_a_samples[SFX_ENEMY_HIT]     = load_sample("sfx/enemy_hit.wav");
	m_a_samples[SFX_ENEMY_DIE]     = load_sample("sfx/enemy_die.wav");
	m_a_samples[SFX_PLAYER_HIT]    = load_sample("sfx/player_hit.wav");
	m_a_samples[SFX_PLAYER_DIE]    = load_sample("sfx/player_die.wav");
	m_a_samples[SFX_HEALTH_PICKUP] = load_sample("sfx/health_pickup.wav");
	m_a_samples[SFX_WALL_HIT]      = load_sample("sfx/wall_hit.wav");
	
	// For each sample, create a voice_list node for it and allocate voices if necessary
	for (int i = 0; i < SFX_SAMPLE_COUNT; i++)
	{
		if (m_a_samples[i] != NULL)
		{
			if (a_voice_counts[i] == 0)
			{
				// No voices need allocating just set up voice_list node
				p_vl = new voice_list;
				if (p_vl != NULL)
				{
					p_vl->type = (SFX_TYPE)i;
					p_vl->p_sample = m_a_samples[i];
					p_vl->voice_id = -1;
					p_vl->vol = a_voice_volumes[i];
					p_vl->p_next = m_p_voice_list;
					m_p_voice_list = p_vl;
				}
				else
					f_result = false;
			}
			else
			{
				// Set up a voice_list node for each voice that needs to be allocated
				for (int voice = 0; voice < a_voice_counts[i]; voice++)
				{
					p_vl = new voice_list;
					if (p_vl != NULL)
					{
						p_vl->type     = (SFX_TYPE)i;
						p_vl->p_sample = m_a_samples[i];
						p_vl->p_next   = m_p_voice_list;
						p_vl->voice_id = allocate_voice(p_vl->p_sample);
						p_vl->vol      = a_voice_volumes[i];
						m_p_voice_list = p_vl;
						if (p_vl->voice_id != -1)
						{
							// Set priority, volume and play mode for voice
							voice_set_priority(p_vl->voice_id, a_voice_priorities[i]);
							voice_set_volume(p_vl->voice_id,   a_voice_volumes[i]);
							voice_set_playmode(p_vl->voice_id, PLAYMODE_PLAY);
						}
						else
							f_result = false;
					}
					else
						f_result = false;
				}
			}
		}
		else
			f_result = false;
	}
	
	return f_result;
}

void c_sfx::play_sfx(SFX_TYPE type)
{
	voice_list *p_vl = m_p_voice_list;
	
	if (!m_f_enabled)
		return;
	
	// Loop through the voice list to find a suitable voice
	while (p_vl != NULL)
	{
		if (p_vl->type == type)
		{
			if (p_vl->voice_id == -1)
			{
				// No voice was allocated, so simply call play_sample()
				play_sample(p_vl->p_sample, p_vl->vol, 128, 1000, 0);
				break;
			}
			else if (voice_get_position(p_vl->voice_id) == -1)
			{
				// The voice is the correct type and not playing, so play it
				voice_start(p_vl->voice_id);
				break;
			}
		}
		p_vl = p_vl->p_next;
	}
}

void c_sfx::set_enabled(bool f_enabled)
{
	m_f_enabled = f_enabled;
}

bool c_sfx::is_enabled(void)
{
	return m_f_enabled;
}
